library ieee;
use ieee.std_logic_1164.all;

entity piii is
	port(
		-- parallel port data and status pins
		ppd:			in	std_logic_vector(7 downto 0);
		pps:			out	std_logic_vector(7 downto 3);

		sw4:			in	std_logic;		-- pushbutton 4

		-- Virtex FPGA pins
		V_tck:		out	std_logic;	-- driver to Virtex JTAG clock
		V_tms:		out	std_logic;	-- driver to Virtex JTAG mode input
		V_tdi:		out	std_logic;	-- driver to Virtex JTAG serial data input
		V_tdo:		in	std_logic;		-- input from Virtex JTAG serial data output
		V_cclk:		out	std_logic;	-- driver to Virtex config clock
		V_prog_n:	out	std_logic;	-- driver to Virtex /PROGRAM pin
		V_done:		in	std_logic;		-- input from Virtex DONE pin
		V_din:		out	std_logic;	-- driver to Virtex config serial data input
		V_m:			out	std_logic_vector(2 downto 0);	-- Virtex config mode pins

		ce_n:			out	std_logic;	-- Flash chip-enable
		reset_n:	out	std_logic;	-- reset for video input and Ethernet chips
		bar:			out	std_logic_vector(9 downto 0)		-- LED bargraph
	);
end piii;

architecture arch of piii is
	constant NO:	std_logic := '0';
	constant YES:	std_logic := '1';
	constant LO:	std_logic := '0';
	constant HI:	std_logic := '1';
	constant SLAVE_SERIAL_MODE: std_logic_vector(2 downto 0) := "111";
begin

	-- disable video/Ethernet and Flash chips
	reset_n		<= LO when V_done=NO else HI; -- disable video/Ethernet until config done
	ce_n			<= HI;	-- disable Flash

	-- the XSV power status is sent back through the parallel port status pin 3
	pps(3)		<= HI;		-- tell the PC that the VCC for the XSV board is OK
	-- the cable is detected by sending data through data pin 6 and returning
	-- it on status pins 5 and 7.  Status pin 7 is also used by the JTAG TDO
	-- pin of the XC9500 CPLD on the XSV Board, so free status pin 7 when
	-- the Virtex is not configured.
	pps(5)		<= ppd(6);
	pps(7)		<= ppd(6) when V_done=YES else 'Z'; -- for iMPACT use "pps(7) <= ppd(6);"

	-- drive the Virtex configuration pins from the parallel port when tristate
	-- control pin (parallel port data pin 3) is low.  Once the Virtex is
	-- configured (DONE=1), use pushbutton SW4 to pull the /PROG pin low to
	-- allow the XSV Board to be programmed again.
	V_m				<= SLAVE_SERIAL_MODE;	-- set Virtex config mode pins
	V_prog_n	<=     ppd(2) when ppd(3)=LO and V_done=NO else sw4;
	V_cclk		<= not ppd(1) when ppd(3)=LO else 'Z';
	V_din			<= not ppd(0) when ppd(3)=LO else 'Z';

	-- drive the Virtex JTAG pins once the Virtex is configured.
	V_tms			<=     ppd(2) when ppd(3)=LO and V_done=YES	else 'Z';
	V_tck			<= not ppd(1) when ppd(3)=LO and V_done=YES	else LO;
	V_tdi			<= not ppd(0) when ppd(3)=LO and V_done=YES	else 'Z';

	-- send the Virtex config. status back to the PC.  Once the Virtex is
	-- configured (DONE=1), the JTAG TDO output is sent back through the
	-- status pin.
	pps(4)		<= V_done and V_tdo when ppd(4)=HI else LO;

	-- display status of Virtex DONE pin on the bargraph LED
	bar(0)		<= V_done;

end arch;

